package main

import (
	"git.randomchars.net/freenitori/log"
	"git.randomchars.net/freenitori/multiplexer"
	"github.com/bwmarrin/discordgo"
	"strconv"
	"strings"
)

func init() {
	m.Route(&multiplexer.Route{
		Pattern:       "ceiling",
		AliasPatterns: []string{"lim", "userlimit", "limit"},
		Description:   "Configure user limit of a channel.",
		Category:      system,
		Handler: func(context *multiplexer.Context) {
			var limit uint8
			if len(context.Fields) != 2 {
				// Return if unexpected amount of arguments.
				context.SendMessage("Expecting new channel user limit.")
				return
			} else if lim, err := strconv.Atoi(context.Fields[1]); err != nil {
				// Return if not numeric.
				context.SendMessage("Invalid argument.")
				return
			} else {
				if lim > 0 && lim <= 99 {
					limit = uint8(lim)
				} else {
					// Return if out of range
					context.SendMessage("Limit out of range.")
					return
				}
			}

			// Get instance
			instance, ok := instancesUser[context.User.ID]
			if !ok {
				context.SendMessage("You do not own any volatile channel.")
				return
			}

			// Perform action
			if !instance.actionPerform("ceiling",
				func() bool {
					// Check if payload is the same as current state
					if instance.Channel.UserLimit == int(limit) {
						context.SendMessage("User limit was not changed.")
						return false
					}

					// Perform change
					if channel, err := context.Session.ChannelEditComplex(instance.Channel.ID,
						&discordgo.ChannelEdit{UserLimit: int(limit), Position: instance.Channel.Position}); !context.HandleError(err) {
						return false
					} else {
						instance.Channel = channel
					}
					return true
				}, context.SendMessage) {
				return
			}

			log.Infof("Set user limit of volatile channel #%s (%s) to %v on behalf of %s#%s (%s).",
				instance.Channel.Name,
				instance.Channel.ID,
				limit,
				instance.Initiator.User.Username,
				instance.Initiator.User.Discriminator,
				instance.Initiator.User.ID)
			context.SendMessage("Action completed successfully.")
		},
	})

	m.Route(&multiplexer.Route{
		Pattern:       "name",
		AliasPatterns: []string{"n", "rename"},
		Description:   "Configure name of a channel.",
		Category:      system,
		Handler: func(context *multiplexer.Context) {
			var name string

			if len(context.Fields) < 2 {
				// Return if unexpected amount of arguments.
				context.SendMessage("Expecting new channel name.")
				return
			} else {
				// Trim out first field since it's normally the command prefix or bot mention
				name = strings.TrimPrefix(
					strings.TrimPrefix(
						strings.TrimPrefix(context.Message.Content,
							context.Multiplexer.Prefix),
						context.Fields[0]),
					" ")

				// Check length
				if len(name) > 32 || len(name) < 2 {
					context.SendMessage("Name length out of range.")
					return
				}

				// Check content
				if !nameRegex.MatchString(name) {
					context.SendMessage("Name is invalid.")
					return
				}
			}

			// Get instance
			instance, ok := instancesUser[context.User.ID]
			if !ok {
				context.SendMessage("You do not own any volatile channel.")
				return
			}

			// Perform action
			if !instance.actionPerform("name",
				func() bool {
					// Check if payload is the same as current state
					if instance.Channel.Name == name {
						context.SendMessage("Channel name was not changed.")
						return false
					}

					// Perform change
					if channel, err := context.Session.ChannelEditComplex(instance.Channel.ID,
						&discordgo.ChannelEdit{Name: name, Position: instance.Channel.Position}); !context.HandleError(err) {
						return false
					} else {
						instance.Channel = channel
					}
					return true
				}, context.SendMessage) {
				return
			}

			log.Infof("Set name of volatile channel #%s (%s) to %v on behalf of %s#%s (%s).",
				instance.Channel.Name,
				instance.Channel.ID,
				name,
				instance.Initiator.User.Username,
				instance.Initiator.User.Discriminator,
				instance.Initiator.User.ID)
			context.SendMessage("Action completed.")
		},
	})

	m.Route(&multiplexer.Route{
		Pattern:       "kick",
		AliasPatterns: []string{"k", "dc"},
		Description:   "Kick an user from channel.",
		Category:      system,
		Handler: func(context *multiplexer.Context) {

			if len(context.Fields) < 2 {
				// Return if unexpected amount of arguments.
				context.SendMessage("Expecting user.")
				return
			}

			// Get member
			member := context.GetMember(context.Fields[1])
			if member == nil {
				context.SendMessage("Expecting member.")
				return
			}

			// Get instance
			instance, ok := instancesUser[context.User.ID]
			if !ok {
				context.SendMessage("You do not own any volatile channel.")
				return
			}

			// Perform action
			if !instance.actionPerform("kick",
				func() bool {
					// Check if member is in the channel
					state, err := context.Session.State.VoiceState(member.GuildID, member.User.ID)
					if err != nil {
						log.Errorf("Error getting state of at guild %s on behalf of member %s, %s",
							member.GuildID, member.User.ID, err)
						return false
					}

					if state.ChannelID != instance.Channel.ID {
						context.SendMessage("The user is not in your volatile channel")
						return false
					}
					// Perform change
					err = context.Session.GuildMemberMove(member.GuildID, member.User.ID, nil)
					if err != nil {
						log.Errorf("Error disconnecting user at guild %s on behalf of member %s, %s",
							member.GuildID, member.User.ID, err)
						return false
					}
					err = context.Session.ChannelPermissionSet(instance.Channel.ID,member.User.ID,1,1024,1048576)
					if err != nil {
						log.Errorf("Error changing permissions for user at guild %s on behalf of member %s, %s",
							member.GuildID, member.User.ID, err)
						return false
					}
					return true
				}, context.SendMessage) {
				return
			}

			log.Infof("Kick user from volatile channel #%s (%s) to %v on behalf of %s#%s (%s).",
				instance.Channel.Name,
				instance.Channel.ID,
				instance.Initiator.User.Username,
				instance.Initiator.User.Discriminator,
				instance.Initiator.User.ID)
			context.SendMessage("Action completed.")
		},
	})
}
