package main

import (
	"git.randomchars.net/freenitori/log"
	"github.com/bwmarrin/discordgo"
	"regexp"
	"sync"
	"time"
)

var nameRegex = regexp.MustCompile(`^[A-Za-z0-9()_-]*$`)

var (
	categoryMap map[string]string
	guildMap    map[string]string
	timeout     time.Duration
)

type chatInstance struct {
	Initiator      *discordgo.Member
	Channel        *discordgo.Channel
	PreviousAction map[string]time.Time
	Members        map[string]bool
	sync.RWMutex
}

var (
	instancesChannel = make(map[string]*chatInstance)
	instancesUser    = make(map[string]*chatInstance)
)

// destroy removes the corresponding Discord voice channel of a chatInstance and deletes the instance.
func (c *chatInstance) destroy() {
	delete(instancesChannel, c.Channel.ID)
	delete(instancesUser, c.Initiator.User.ID)
	if _, err := session.ChannelDelete(c.Channel.ID); err != nil {
		log.Errorf("Error destroying channel %s initiated by user %s in guild %s, %s",
			c.Channel.ID, c.Initiator.User.ID, c.Initiator.GuildID, err)
	}
	log.Infof("Chat instance with channel %s initiated by user %s#%s (%s) destroyed.",
		c.Channel.ID, c.Initiator.User.Username, c.Initiator.User.Discriminator, c.Initiator.User.ID)
}

// actionFree checks if actions are free in chatInstance.
func (c *chatInstance) actionFree(variant string) bool {
	p, ok := c.PreviousAction[variant]
	if !ok {
		return true
	}

	return p.Add(timeout).Before(time.Now())
}

// actionRegister registers an action in chatInstance.
func (c *chatInstance) actionRegister(variant string) {
	c.PreviousAction[variant] = time.Now()
}

// newInstance creates a new chatInstance on behalf of a discordgo.Member.
func newInstance(member *discordgo.Member) *chatInstance {
	instance := setupInstance(member)
	if instance != nil {
		instancesChannel[instance.Channel.ID] = instance
		instancesUser[instance.Initiator.User.ID] = instance
		return instance
	}
	return nil
}

// setupInstance sets up a new chatInstance on behalf of a discordgo.Member.
func setupInstance(member *discordgo.Member) *chatInstance {
	channelID := guildMap[member.GuildID]
	if channelID == "" {
		log.Debugf("Instance setup skipped on empty channel ID for guild %s member %s.", member.GuildID, member.User.ID)
		return nil
	}

	parentID := categoryMap[channelID]
	if parentID == "" {
		log.Debug("Instance setup skipped on empty parent ID.")
		return nil
	}

	channel, err := session.GuildChannelCreateComplex(member.GuildID, discordgo.GuildChannelCreateData{
		Name:      member.User.Username + "#" + member.User.Discriminator,
		Type:      discordgo.ChannelTypeGuildVoice,
		Topic:     "Volatile channel created by " + member.User.ID,
		Bitrate:   64000,
		UserLimit: config.DefaultUserLimit,
		ParentID:  parentID,
	})
	if err != nil {
		log.Errorf("Error creating channel on guild %s on behalf of member %s, %s",
			member.GuildID, member.User.ID, err)
		return nil
	}

	return &chatInstance{
		Initiator:      member,
		Channel:        channel,
		PreviousAction: make(map[string]time.Time),
		Members:        make(map[string]bool),
	}
}

// actionPerform performs an action.
func (c *chatInstance) actionPerform(variant string, action func() bool, sendMessage func(message string) *discordgo.Message) bool {
	// Lock the instance for write
	c.Lock()
	defer c.Unlock()

	// Check if action is free
	if !c.actionFree(variant) {
		sendMessage("Not so fast! Please try again later.")
		return false
	}

	// Perform action
	ok := action()

	// Register action
	if ok {
		c.actionRegister(variant)
	}

	return ok
}
