package main

import (
	"flag"
	"fmt"
	"git.randomchars.net/freenitori/log"
	"git.randomchars.net/freenitori/multiplexer"
	"github.com/bwmarrin/discordgo"
	"github.com/sirupsen/logrus"
	"os"
	"os/signal"
	"syscall"
)

var (
	session *discordgo.Session
	m       = multiplexer.New()
	system  = multiplexer.NewCategory("System", "System-related utilities.")
	verbose bool
)

func init() {
	flag.BoolVar(&verbose, "v", false, "Start up with debug logging.")
}

func main() {
	flag.Parse()
	parse()

	if verbose {
		log.SetLevel(logrus.DebugLevel)
	} else {
		log.SetLevel(logrus.InfoLevel)
	}

	// Set discordgo log handler
	discordgo.Logger = func(msgL, _ int, format string, a ...interface{}) {
		var level logrus.Level
		switch msgL {
		case discordgo.LogDebug:
			level = logrus.DebugLevel
		case discordgo.LogInformational:
			level = logrus.InfoLevel
		case discordgo.LogWarning:
			level = logrus.WarnLevel
		case discordgo.LogError:
			level = logrus.ErrorLevel
		}
		log.Instance.Log(level, fmt.Sprintf(format, a...))
	}

	// Set command not found handler
	multiplexer.NoCommandMatched = func(context *multiplexer.Context) {
		return
	}

	// Configure session
	if s, err := discordgo.New(); err != nil {
		log.Fatalf("Error while creating session, %s", err)
		os.Exit(1)
	} else {
		session = s
	}
	session.UserAgent = "DiscordBot (voice-bot)"
	session.Token = "Bot " + config.Token
	session.ShouldReconnectOnError = true
	session.Identify.Intents = discordgo.IntentsAll
	session.State.TrackVoice = true

	// Open session
	func() {
	open:
		if err := session.Open(); err != nil {
			if session.Identify.Intents == discordgo.IntentsAll {
				log.Warnf("Wasn't able to start with full intents, some stuff might not work (%s)", err)
				session.Identify.Intents = discordgo.IntentsAllWithoutPrivileged
				goto open
			}

			log.Fatalf("Error while opening session, %s", err)
			os.Exit(1)
		}
	}()

	// Execute late config options
	configLate()

	// Setup multiplexer
	m.SessionRegisterHandlers(session)
	m.Prefix = config.Prefix
	m.Categories = append(m.Categories, system)

	// Output message
	log.Infof("Logged in as %s#%s (%s).",
		session.State.User.Username,
		session.State.User.Discriminator,
		session.State.User.ID)
	if application, err := session.Application("@me"); err != nil {
		log.Fatalf("Unable to get application, %s", err)
		os.Exit(1)
	} else {
		log.Infof("Invite URL: https://discord.com/oauth2/authorize?client_id=%s&scope=bot&permissions=8", application.ID)
	}

	// Signal handling
	signalChannel := make(chan os.Signal, 1)
	signal.Notify(signalChannel, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, os.Interrupt, os.Kill)
	var exit int
	func() {
		for {
			currentSignal := <-signalChannel
			switch currentSignal {
			case os.Interrupt:
				exit = 0
				println()
				log.Info("Gracefully exiting.")
				return
			default:
				exit = 0
				log.Info("Gracefully exiting.")
				return
			}
		}
	}()

	switch exit {
	case 0:
		cleanup()
	default:
		os.Exit(exit)
	}
}
